/* Copyright 2019
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdint.h>
#include <avr/io.h>
#include "matrix.h"
#include "quantum.h"

static void init_pins(void)
{
  setPinOutput(B7);
  setPinOutput(D0);
  setPinOutput(D1);
  setPinOutput(D2);

  setPinInputHigh(D5);
  setPinInputHigh(D6);
  setPinInputHigh(D7);
  setPinInputHigh(E0);
  setPinInputHigh(E1);
  setPinInputHigh(C0);
  setPinInputHigh(C1);
  setPinInputHigh(C2);
}

static void select_row(uint8_t row)
{
  int const pin0 = 0b1000;
  int const pin1 = 0b0100;
  int const pin2 = 0b0010;
  int const pin3 = 0b0001;

  writePinHigh(D2);
  writePinHigh(D1);
  writePinHigh(D0);
  writePinHigh(B7);

  if((~row)&pin0)
  {
    writePinLow(D2);
  }
  if((~row)&pin1)
  {
    writePinLow(D1);
  }
  if((~row)&pin2)
  {
    writePinLow(D0);
  }
  if((~row)&pin3)
  {
    writePinLow(B7);
  }
}

static matrix_row_t read_cols(void)
{
  matrix_row_t state = 0;

  state |= readPin(D5) << 0;
  state |= readPin(D6) << 1;
  state |= readPin(D7) << 2;
  state |= readPin(E0) << 3;
  state |= readPin(E1) << 4;
  state |= readPin(C0) << 5;
  state |= readPin(C1) << 6;
  state |= readPin(C2) << 7;

  return ~state;
}

static bool read_cols_on_row(matrix_row_t current_matrix[], uint8_t current_row)
{
  matrix_row_t last_row_value = current_matrix[current_row];

  current_matrix[current_row] = 0;

  select_row(current_row);
  wait_us(30);

  current_matrix[current_row] = read_cols();

  return (last_row_value != current_matrix[current_row]);
}

void matrix_init_custom(void)
{
  init_pins();
}

bool matrix_scan_custom(matrix_row_t current_matrix[])
{
  bool changed = false;
  for (uint8_t current_row = 0; current_row < MATRIX_ROWS; current_row++)
  {
    changed |= read_cols_on_row(current_matrix, current_row);
  }

  return changed;
}
